import sys, os
import unittest

os.environ["TF_NUM_INTEROP_THREADS"] = "8"
os.environ["TF_NUM_INTRAOP_THREADS"] = "8"
os.environ["ITK_GLOBAL_DEFAULT_NUMBER_OF_THREADS"] = "8"

import tempfile
import shutil
import tensorflow as tf
import antspymm
import antspyt1w
import antspynet
import ants
from dipy.io.gradients import read_bvals_bvecs                                 

testingClass = unittest.TestCase( )
islocal = False
id1 = "I1499279_Anon_20210819142214_5"
id2 = "I1499337_Anon_20210819142214_6"
img1 = ants.image_read( antspymm.get_data( id1, target_extension=".nii.gz") )
img2 = ants.image_read( antspymm.get_data( id2, target_extension=".nii.gz") )

bvec = antspymm.get_data( id1, target_extension=".bvec")
bval = antspymm.get_data( id1, target_extension=".bval")
dd = antspymm.dipy_dti_recon( img1, bval, bvec, verbose=True )
# scrub example
bvalr, bvecr = read_bvals_bvecs( bval, bvec )
img1_sc, bval_sc, bvec_sc = antspymm.scrub_dwi( img1, bvalr, bvecr, threshold=0.01, verbose=True )

# Create a temp directory using tempfile
with tempfile.TemporaryDirectory() as tmpdir:
    # Define file prefixes and paths
    prefix = os.path.join(tmpdir, 'temp')
    bval_path = prefix + '.bval'
    bvec_path = prefix + '.bvec'
    fa_path_0 = os.path.join(tmpdir, 'temp0.nii.gz')
    fa_path_1 = os.path.join(tmpdir, 'temp1.nii.gz')

    # Write bvals and bvecs
    antspymm.write_bvals_bvecs(bvals=bval_sc, bvecs=bvec_sc, prefix=prefix)

    # DTI reconstruction
    ee = antspymm.dipy_dti_recon(img1_sc, bval_path, bvec_path, verbose=True)

    # Write output images for comparison
    # ants.image_write(dd['FA'], fa_path_0)
    # ants.image_write(ee['FA'], fa_path_1)

    # Temporary files will be cleaned up automatically when the block ends


##################################################################
# This script processes neuroimaging data in NRG format using antspymm.
# NRG format: https://github.com/stnava/biomedicalDataOrganization
##################################################################

import os
import glob
import numpy as np
import pandas as pd
import ants
import antspymm

# ---------------------------
# Configuration
# ---------------------------
nthreads = "8"
os.environ["TF_NUM_INTEROP_THREADS"] = nthreads
os.environ["TF_NUM_INTRAOP_THREADS"] = nthreads
os.environ["ITK_GLOBAL_DEFAULT_NUMBER_OF_THREADS"] = nthreads
os.environ["OPENBLAS_NUM_THREADS"] = nthreads
os.environ["MKL_NUM_THREADS"] = nthreads

# Base directory in NRG format
out_dir = os.path.expanduser("./example/processedX/")
base_dir = os.path.expanduser("./nrgdata/data/")
projid="PPMI"
participant_num = "182341"
participant_date = "20230111"
t1id="1681010"
participant_path = os.path.join(base_dir, projid, participant_num, participant_date)

# ---------------------------
# Locate image files
# ---------------------------
def get_first_file(pattern, required=True):
    files = glob.glob(pattern)
    if required and not files:
        raise FileNotFoundError(f"No files found for pattern: {pattern}")
    return files[0] if files else None

t1_file = get_first_file(os.path.join(participant_path, "T1w/"+t1id+"/*.nii.gz"))
flair_file = get_first_file(os.path.join(participant_path, "T2Flair/*/*.nii.gz"))
dti_files = glob.glob(os.path.join(participant_path, "DTI*/*/*.nii.gz"))
rsf_files = glob.glob(os.path.join(participant_path, "rsfMRI*/*/*1681014*.nii.gz"))+glob.glob(os.path.join(participant_path, "rsfMRI*/*/*1681015*.nii.gz"))
nm_files = glob.glob(os.path.join(participant_path, "NM2DMT/*/*.nii.gz"))

# ---------------------------
# Generate multi-modal dataframe
# ---------------------------
mm_df = antspymm.generate_mm_dataframe(
    projectID='PPMI',
    subjectID=participant_num,
    date=participant_date,
    imageUniqueID=t1id,
    modality="T1w",
    source_image_directory=base_dir,
    output_image_directory=out_dir,
    t1_filename=t1_file,
    flair_filename=flair_file,
    rsf_filenames=rsf_files,
    dti_filenames=dti_files,
    nm_filenames=nm_files
)

# Drop NA columns and run analysis
mm_df_clean = mm_df.dropna(axis=1)
print(mm_df_clean)

# Run antspymm pipeline
mm_result = antspymm.mm_csv(mm_df_clean)


import numpy as np
import ants
import os
from dipy.io.gradients import read_bvals_bvecs
from scipy.stats import pearsonr
import antspymm
import matplotlib.pyplot as plt
nt = 2
# amount or rotation around x, y, z-axis
degrotx = 15
degroty = 15
degrotz = 15


def mean_rgb_correlation(img1, img2, mask):
    """
    Compute the mean correlation between two RGB images.

    Parameters
    ----------
    img1 : np.ndarray
        First RGB image as a (H, W, 3) NumPy array.
    img2 : np.ndarray
        Second RGB image as a (H, W, 3) NumPy array.

    Returns
    -------
    float
        Mean Pearson correlation across the three RGB channels.
    """
    if img1.shape != img2.shape:
        raise ValueError("Input images must have the same shape.")
    correlations = []
    img1c = ants.split_channels(img1)
    img2c = ants.split_channels(img2)   
    for c in range(3):  # R, G, B
        x = extract_masked_values( img1c[c], mask)
        y = extract_masked_values( img2c[c], mask)
        if np.std(x) == 0 or np.std(y) == 0:
            corr = 0.0  # Handle flat images
        else:
            corr, _ = pearsonr(x, y)
        correlations.append(corr)
    return np.mean(correlations)


def plot_correlation(x, y, xlabel="Image 1", ylabel="Image 2", title_prefix="Correlation", point_alpha=0.3):
    """
    Plot the correlation between two 1D arrays and display Pearson r.

    Parameters
    ----------
    x : array-like
        First set of values.
    y : array-like
        Second set of values.
    xlabel : str
        Label for the x-axis.
    ylabel : str
        Label for the y-axis.
    title_prefix : str
        Title prefix, will append r value.
    point_alpha : float
        Transparency of scatter plot points.

    Returns
    -------
    float
        Pearson correlation coefficient (r).
    """
    x = np.asarray(x)
    y = np.asarray(y)

    if x.shape != y.shape:
        raise ValueError("Inputs must have the same shape.")

    r, _ = pearsonr(x, y)

    plt.figure(figsize=(6, 6))
    plt.scatter(x, y, s=1, alpha=point_alpha, color='blue')
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.title(f"{title_prefix} (r = {r:.3f})")
    plt.plot([x.min(), x.max()], [x.min(), x.max()], 'r--', linewidth=1)  # Identity line
    plt.grid(True)
    plt.axis('equal')
    plt.tight_layout()
    plt.show()

    return r

def read_bvecs_rotated(bvec_file, rotmat):
    bvecs = np.loadtxt(bvec_file)
    if bvecs.shape[0] != 3:
        bvecs = bvecs.T
    rotated_bvecs = (rotmat @ bvecs).T
    return rotated_bvecs

def broadcast_bvecs_voxelwise(rotated_bvecs, shape):
    return np.broadcast_to(rotated_bvecs, shape + rotated_bvecs.shape).copy()

def extract_masked_values(image, mask):
    return image.numpy()[mask.numpy() > 0]

# def test_efficient_dwi_fit_voxelwise_rotation_consistency():
if True:
    print("simple test for rotation consistency of efficient_dwi_fit_voxelwise")
    ex_path = os.path.expanduser( "~/.antspyt1w/" )
    ex_path_mm = os.path.expanduser( "~/.antspymm/" )
    JHU_atlas = ants.image_read( ex_path + 'JHU-ICBM-FA-1mm.nii.gz' ) # Read in JHU atlas
    JHU_labels = ants.image_read( ex_path + 'JHU-ICBM-labels-1mm.nii.gz' ) # Read in JHU labels
    #### Load in data ####
    print("Load in subject data ...")
    lrid = ex_path_mm + "I1499279_Anon_20210819142214_5"
    rlid = ex_path_mm + "I1499337_Anon_20210819142214_6"
    t1id = ex_path_mm + "t1_rand.nii.gz"
    # Load paths
    print("📁 Loading subject data...")
    lrid = os.path.join(ex_path_mm, "I1499279_Anon_20210819142214_5")
    img_LR_in = ants.image_read(lrid + '.nii.gz')
    img_LR_in_avg = ants.get_average_of_timeseries( img_LR_in )
    mask = img_LR_in_avg.get_mask()
    
    bvals, bvecs = read_bvals_bvecs(lrid + '.bval', lrid + '.bvec')
    bvecs = np.asarray(bvecs)
    shape = img_LR_in.shape[:3]

    print("🧠 Running baseline (unrotated) fit...")
    bvecs_5d_orig = np.broadcast_to(bvecs, shape + bvecs.shape).copy()
    if not "FA_orig" in globals():
        FA_orig, MD_orig, RGB_orig = antspymm.efficient_dwi_fit_voxelwise(
            imagein=img_LR_in,
            maskin=mask,
            bvals=bvals,
            bvecs_5d=bvecs_5d_orig,
            model_params={},
            bvals_to_use=None,
            num_threads=nt,
            verbose=False
        )

    print("🔄 Applying known rotation...")
    # maxtrans = 10.0
    # rotator = ants.contrib.RandomRotate3D((-maxtrans, maxtrans), reference=img_LR_in_avg)
    rotator=ants.contrib.Rotate3D(rotation=(degrotx,degroty,degrotz), reference=img_LR_in_avg )
    rotation = rotator.transform()
    img_rotated_ref = ants.apply_ants_transform_to_image(rotation, img_LR_in_avg, reference=img_LR_in_avg)
    img_rotated = antspymm.timeseries_transform(rotation, img_LR_in, reference=img_LR_in_avg)
    mask_rotated = ants.apply_ants_transform_to_image(rotation, mask, reference=img_LR_in_avg, interpolation='nearestNeighbor')
    # note: we apply the inverse rotation to the bvecs
    # i.e. if we register A to B and get R then apply R_inv to bvecs
    rotmat = ants.get_ants_transform_parameters(rotation.invert()).reshape((4, 3))[:3, :3]
    bvecs_rotated = read_bvecs_rotated(lrid + '.bvec', rotmat)
    bvecs_5d_rot = broadcast_bvecs_voxelwise(bvecs_rotated, shape)

    print("🧠 Running rotated fit...")
    if not "FA_rot" in globals():
        FA_rot, MD_rot, RGB_rot = antspymm.efficient_dwi_fit_voxelwise(
            imagein=img_rotated,
            maskin=mask_rotated,
            bvals=bvals,
            bvecs_5d=bvecs_5d_rot,
            model_params={},
            bvals_to_use=None,
            num_threads=nt,
            verbose=False
        )

    FA_rot_back = ants.apply_ants_transform_to_image(
        rotation.invert(),
        FA_rot,
        reference = img_LR_in_avg,
        )
    MD_rot_back = ants.apply_ants_transform_to_image(
        rotation.invert(),
        MD_rot,
        reference = img_LR_in_avg,
        )
    print("📊 Comparing results...")
    maske=ants.iMath(mask,'ME',2)
    # smoothing simulates double interpolation
    FA_origs = ants.smooth_image(FA_orig, 1.25)
    MD_origs = ants.smooth_image(MD_orig, 1.25)
    fa1 = extract_masked_values(FA_origs, maske)
    fa2 = extract_masked_values(FA_rot_back, maske)
    md1 = extract_masked_values(MD_origs, maske)
    md2 = extract_masked_values(MD_rot_back, maske)

    fa_corr, _ = pearsonr(fa1, fa2)
    md_corr, _ = pearsonr(md1, md2)

#    plot_correlation( fa1, fa2 )

    print(f"✅ FA correlation (original vs rotated): {fa_corr:.4f}")
    print(f"✅ MD correlation (original vs rotated): {md_corr:.4f}")
    
    assert fa_corr > 0.80, "FA correlation too low"
    assert md_corr > 0.80, "MD correlation too low"

    print("🎉 Test passed: model is rotation-consistent with voxelwise bvecs.")

    print("This shows that the simulation is effective.")
    print("Now use the simulated data to test the distortion correction...")
#    ants.image_write( RGB_orig, '/tmp/temp0rgb.nii.gz' )
#    ants.image_write( RGB_rot, '/tmp/temp1rgb.nii.gz' )


    # now we have to map the img_rotated and its bvec_rotated partner 
    # as we would with a generic distortion correction framework. 
    # first --- write the transform to a file
    rotationinv = rotation.invert()

    import tempfile
    with tempfile.TemporaryDirectory() as tempdir:
        rotmatfile = os.path.join(tempdir, "rotation.mat")
        compositefile = os.path.join(tempdir, "composite")

        # Write the rotation transform
        ants.write_transform(rotationinv, rotmatfile)

        # Run registration if not already done
        if "reg" not in globals():
            reg = ants.registration(FA_orig, FA_rot, 'SyN', initial_transform=rotmatfile)

        # Apply the transformation using a temporary composite path
        comptx = ants.apply_transforms(
            img_LR_in_avg,
            img_rotated_ref,
            reg['fwdtransforms'],
            interpolator='linear',
            compose=compositefile,
            verbose=True
        )
        mydef=ants.image_read(comptx)
        mydefgrad = antspymm.deformation_gradient_optimized( mydef, 
                to_rotation=False, to_inverse_rotation=True )
        bvecsRLw = antspymm.generate_voxelwise_bvecs( bvecs_rotated, mydefgrad, transpose=False )
        mywarp = ants.transform_from_displacement_field( mydef )
        img_rotated_avg = ants.get_average_of_timeseries(  img_rotated )
        img_w = antspymm.timeseries_transform(mywarp, img_rotated, reference=img_rotated_avg )

        correlations = []
        labels = ["NoBvecReo", "BvecReo"]

        RGB_origs = ants.smooth_image(RGB_orig, 1.0)

        for label, bv in zip(labels, [bvecs_5d_rot, bvecsRLw]):
            FA_w, MD_w, RGB_w = antspymm.efficient_dwi_fit_voxelwise(
                imagein=img_w,
                maskin=ants.get_mask(ants.get_average_of_timeseries(img_w)),
                bvals=bvals,
                bvecs_5d=bv,
                model_params={},
                bvals_to_use=None,
                num_threads=nt,
                verbose=False
            )
            # Optional: write RGB images to temporary files for inspection
            rgb_orig_path = os.path.join(tempdir, "RGB_orig.nii.gz")
            rgb_warped_path = os.path.join(tempdir, f"RGB_warped_{label}.nii.gz")

            ants.image_write(RGB_origs, rgb_orig_path)
            ants.image_write(RGB_w, rgb_warped_path)

            print(f"📝 Saved RGB_orig to: {rgb_orig_path}")
            print(f"📝 Saved RGB_warped ({label}) to: {rgb_warped_path}")

            compmask = ants.get_mask(ants.get_average_of_timeseries(img_w))
            compmask = ants.iMath(compmask, 'ME', 2)

            corr = mean_rgb_correlation(RGB_origs, RGB_w, compmask)
            print(f"{label:10s} correlation: {corr:.4f}")
            correlations.append(corr)

# Compare the two correlations
if len(correlations) == 2:
    print("\nComparison Result:")
    if correlations[1] > correlations[0]:
        print("✅ BvecReo gives higher correlation than NoBvecReo.")
    else:
        print("❌ NoBvecReo gives equal or higher correlation than BvecReo.")


import numpy as np
import ants
import os
from dipy.io.gradients import read_bvals_bvecs
from scipy.stats import pearsonr
import antspymm
nt = 2

import numpy as np
from scipy.stats import pearsonr

def read_bvecs_rotated(bvec_file, rotmat):
    bvecs = np.loadtxt(bvec_file)
    if bvecs.shape[0] != 3:
        bvecs = bvecs.T
    rotated_bvecs = (rotmat @ bvecs).T
    return rotated_bvecs

def mean_rgb_correlation(img1, img2, mask):
    """
    Compute the mean correlation between two RGB images.

    Parameters
    ----------
    img1 : np.ndarray
        First RGB image as a (H, W, 3) NumPy array.
    img2 : np.ndarray
        Second RGB image as a (H, W, 3) NumPy array.

    Returns
    -------
    float
        Mean Pearson correlation across the three RGB channels.
    """
    if img1.shape != img2.shape:
        raise ValueError("Input images must have the same shape.")
    correlations = []
    img1c = ants.split_channels(img1)
    img2c = ants.split_channels(img2)   
    for c in range(3):  # R, G, B
        x = extract_masked_values( img1c[c], mask)
        y = extract_masked_values( img2c[c], mask)
        if np.std(x) == 0 or np.std(y) == 0:
            corr = 0.0  # Handle flat images
        else:
            corr, _ = pearsonr(x, y)
        correlations.append(corr)
    return np.mean(correlations)

import numpy as np
import ants

def mean_rgb_mae(img1, img2, mask):
    """
    Compute the mean absolute error (MAE) between two RGB images.

    Parameters
    ----------
    img1 : np.ndarray
        First RGB image as a (H, W, 3) NumPy array.
    img2 : np.ndarray
        Second RGB image as a (H, W, 3) NumPy array.
    mask : ants.ANTsImage
        Binary mask defining valid pixels for error calculation.

    Returns
    -------
    float
        Mean absolute error across the three RGB channels.
    """
    if img1.shape != img2.shape:
        raise ValueError("Input images must have the same shape.")
    
    mae_values = []
    img1c = ants.split_channels(img1)
    img2c = ants.split_channels(img2)   
    
    for c in range(3):  # R, G, B
        x = extract_masked_values(img1c[c], mask)
        y = extract_masked_values(img2c[c], mask)
        mae = np.mean(np.abs(x - y))
        mae_values.append(mae)
    
    return np.mean(mae_values)

def extract_masked_values(image, mask):
    return image.numpy()[mask.numpy() > 0]

import numpy as np

def verify_unit_bvecs(bvecs, tol=1e-5):
    """
    Verifies that each b-vector has unit norm within a tolerance.
    
    Parameters
    ----------
    bvecs : array-like, shape (N, 3)
        Array of b-vectors, one per diffusion direction.
    tol : float
        Tolerance for unit norm.

    Returns
    -------
    is_unit : np.ndarray, shape (N,)
        Boolean array indicating if each b-vector is unit norm.
    norms : np.ndarray, shape (N,)
        Norms of each b-vector.
    """
    bvecs = np.asarray(bvecs)
    norms = np.linalg.norm(bvecs, axis=1)
    is_unit = np.abs(norms - 1) < tol
    return is_unit, norms

# def test_efficient_dwi_fit_voxelwise_distortion_correction():
if True:
    print("simple test for distortion_correction consistency of efficient_dwi_fit_voxelwise")
    ex_path = os.path.expanduser( "~/.antspyt1w/" )
    ex_path_mm = os.path.expanduser( "~/.antspymm/" )
    #### Load in data ####
    print("Load in subject data ...")
    lrid = "I1499279_Anon_20210819142214_5"
    rlid = "I1499337_Anon_20210819142214_6"
    # Load paths
    print("📁 Loading subject LR data...")
    lrid = os.path.join(ex_path_mm, lrid )
    img_LR_in = ants.image_read(lrid + '.nii.gz')
    img_LR_in_avg = ants.get_average_of_timeseries( img_LR_in )
    mask = img_LR_in_avg.get_mask()
    bvals, bvecs = read_bvals_bvecs(lrid + '.bval', lrid + '.bvec')
    bvecs = np.asarray(bvecs)
    shape = img_LR_in.shape[:3]

    print("📁 Loading subject RL data...")
    rlid = os.path.join(ex_path_mm, rlid )
    img_RL_in = ants.image_read(rlid + '.nii.gz')
    bvalsRL, bvecsRL = read_bvals_bvecs(rlid + '.bval', rlid + '.bvec')

    img_RL_in_avg = ants.get_average_of_timeseries( img_RL_in )
    maskRL = img_RL_in_avg.get_mask()

    print("🧠 Running baseline LR fit...")
    bvecs_5d_orig = np.broadcast_to(bvecs, shape + bvecs.shape).copy()
    if not "FA_orig" in globals():
        FA_orig, MD_orig, RGB_orig = antspymm.efficient_dwi_fit_voxelwise(
            imagein=img_LR_in,
            maskin=mask,
            bvals=bvals,
            bvecs_5d=bvecs_5d_orig,
            model_params={},
            bvals_to_use=None,
            num_threads=nt,
            verbose=False
        )

    bvecs_5d_origRL = np.broadcast_to(np.asarray(bvecsRL), shape + bvecsRL.shape).copy()
    if not "FA_origRL" in globals():
        FA_origRL, MD_origRL, RGB_origRL = antspymm.efficient_dwi_fit_voxelwise(
            imagein=img_RL_in,
            maskin=maskRL,
            bvals=bvalsRL,
            bvecs_5d=bvecs_5d_origRL,
            model_params={},
            bvals_to_use=None,
            num_threads=nt,
            verbose=False
        )

    print("dist corr")
    if not "mytx" in globals():
        mytx = ants.registration( FA_orig, FA_origRL, 'SyNBold' )
        mytx2 = ants.apply_transforms(FA_orig, FA_origRL, mytx['fwdtransforms'],
            interpolator='linear', imagetype=0, compose='/tmp/comptx' )
        print( mytx2 )

    print("🔄 now with distortion correction...")
    mydef = ants.image_read( mytx2 )
    mywarp = ants.transform_from_displacement_field( mydef )
    img_w = antspymm.timeseries_transform(mywarp, img_RL_in, reference=img_LR_in_avg)
    mask_w = ants.apply_ants_transform_to_image(mywarp, maskRL, reference=img_LR_in_avg, interpolation='nearestNeighbor')
    print("🧠 Running warped fit...")
    if not "FA_w" in globals():
        bvecsRL = np.asarray(bvecsRL)
        mydefgrad = antspymm.deformation_gradient_optimized( mydef, 
            to_rotation=False, to_inverse_rotation=True )
        bvecsRLw = antspymm.generate_voxelwise_bvecs( bvecsRL, mydefgrad, transpose=False )
        FA_w, MD_w, RGB_w = antspymm.efficient_dwi_fit_voxelwise(
            imagein=img_w,
            maskin=mask_w,
            bvals=bvalsRL,
            bvecs_5d=bvecsRLw,
            model_params={},
            bvals_to_use=None,
            num_threads=nt,
            verbose=False
        )

    if not "FA_w2" in globals():
        bvecsRL = np.asarray(bvecsRL)
        FA_w2, MD_w2, RGB_w2 = antspymm.efficient_dwi_fit_voxelwise(
            imagein=img_w,
            maskin=mask_w,
            bvals=bvalsRL,
            bvecs_5d=bvecs_5d_origRL,
            model_params={},
            bvals_to_use=None,
            num_threads=nt,
            verbose=False
        )

    fff = mean_rgb_correlation
    print("📊 Comparing results...")
    maskJoined = ants.threshold_image( mask + mask_w, 1.05, 2.0 )
    maske=ants.iMath(maskJoined,'ME',3)
    fa_corr = fff( RGB_orig, RGB_w, maske )
    print(f"✅ FA correlation (original vs distortion corrected): {fa_corr:.4f}")

    fa_corrX = fff( RGB_orig, RGB_w2, maske )
    print(f"✅ FA correlation (original vs distortion corrected global recon): {fa_corrX:.4f}")

    RGB_origRLc = ants.split_channels(RGB_origRL)
    for c in range(3):
        RGB_origRLc[c] = ants.apply_ants_transform_to_image(
            mywarp, RGB_origRLc[c], 
            reference=img_LR_in_avg, interpolation='linear'
        )

    fa_corrY = fff( RGB_orig, ants.merge_channels(RGB_origRLc), maske )
    print(f"✅ FA correlation (original vs warped RGB global recon): {fa_corrY:.4f}")

#    assert fa_corr > 0.80, "FA correlation too low"

    print("🎉 Test passed: model is distortion-consistent.")

# ants.image_write( FA_orig, '/tmp/xxx.nii.gz' )
# ants.image_write( FA_w, '/tmp/yyy.nii.gz' )
#
# Example usage:
# test_efficient_dwi_fit_voxelwise_distortion_correction()



import sys, os
import unittest

os.environ["TF_NUM_INTEROP_THREADS"] = "8"
os.environ["TF_NUM_INTRAOP_THREADS"] = "8"
os.environ["ITK_GLOBAL_DEFAULT_NUMBER_OF_THREADS"] = "8"

import tempfile
import shutil
import tensorflow as tf
import antspymm
import antspyt1w
import antspynet
import ants
import numpy as np
from scipy.stats import median_abs_deviation
import math

def replicate_list(user_list, target_size):
    # Calculate the number of times the list should be replicated
    replication_factor = target_size // len(user_list)
    # Replicate the list and handle any remaining elements
    replicated_list = user_list * replication_factor
    remaining_elements = target_size % len(user_list)
    replicated_list += user_list[:remaining_elements]
    return replicated_list

def one_hot_encode(char_list):
    unique_chars = list(set(char_list))
    encoding_dict = {char: [1 if char == c else 0 for c in unique_chars] for char in unique_chars}
    encoded_matrix = np.array([encoding_dict[char] for char in char_list])
    return encoded_matrix

islocal = False
bdir=os.path.expanduser('~/.antspymm/')
t1fn = bdir + 'sub-1001-0462_run-001_t1.nii.gz'
ffn= bdir + 'sub-1001-0462_run-001_perf.nii.gz'
fmri = ants.image_read( ffn )
verbose=True
#dkt
t1head = ants.image_read( t1fn ).n3_bias_field_correction( 8 ).n3_bias_field_correction( 4 )
t1bxt = antspynet.brain_extraction( t1head, 't1' ).threshold_image( 0.3, 1.0 )
t1 = t1bxt * t1head

if not 't1seg' in globals():
  t1seg = antspynet.deep_atropos( t1head )

t1segmentation = t1seg['segmentation_image']

if not 'dkt' in globals():
  dkt = antspynet.desikan_killiany_tourville_labeling( t1head )
#################

#################
type_of_transform='Rigid'
tc='alternating'
fmri_template, hlinds = antspymm.loop_timeseries_censoring( fmri, 0.1 )
fmri_template = ants.get_average_of_timeseries( fmri_template )
print("do perf")
olthresh=0.2
perf = antspymm.bold_perfusion( 
  fmri, 
  t1head, 
  t1, 
  t1segmentation, dkt, nc=4, 
  type_of_transform=type_of_transform,
  spa=(0.,0.,0.,0.),
  outlier_threshold=olthresh, add_FD_to_nuisance=False, verbose=True )

# ants.image_write( ants.iMath( perf['perfusion'], "Normalize" ), '/tmp/temp.nii.gz' )
# ants.image_write( perf['motion_corrected'], '/tmp/temp2.nii.gz' )
ants.plot( ants.iMath( perf['perfusion'], "Normalize" ), axis=2, crop=True )
